import logging

from django.contrib.auth.models import AnonymousUser

from rest_framework.authentication import BaseAuthentication as BA
from rest_framework.authentication import SessionAuthentication as SA
from rest_framework_jwt.authentication import JSONWebTokenAuthentication as JA


logger = logging.getLogger(__name__)


class UserAuthMixin(object):
    def authenticate(self, request):
        result = super().authenticate(request)

        if result is None:
            return

        user, token = result
        logger.debug("%s: authenticate as %s",
            self.__class__.__name__, user.username or '<anon>')

        # Authenticate user in database session
        request.db.login(user)

        return (user, token)


class AnonymousAuthenticationBase(BA):
    def authenticate(self, request):
        return AnonymousUser(), None


class JWTAuthentication(UserAuthMixin, JA): pass
class SessionAuthentication(UserAuthMixin, SA): pass
class AnonymousAuthentication(UserAuthMixin, AnonymousAuthenticationBase): pass
