import logging

from gargantext.core.db import Session


logger = logging.getLogger('gargantext')


class DatabaseSessionMiddleware(object):
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        logger.debug('---8<----- DatabaseSession: start ----------')

        # Create database session and attach it to the current request instance
        session = request.db = Session()

        # Render view: if any exception occurs while doing this, it will be
        # handled by django rest framework, so we don't need to care about
        # exceptions here.
        response = self.get_response(request)

        # Commit to database if no error happened
        if not session.is_rollbacked:
            session.commit()

        # Finally, close the session
        session.close()

        logger.debug('---------- DatabaseSession: end ------->8---')

        return response
